# Copyright (c) 2010 The Foundry Visionmongers Ltd.  All Rights Reserved.
import platform
import sys
import os.path
import re
import thread
import nuke
import subprocess
import nukescripts
import nukescripts.flipbooking as flipbooking
import time
################################################################################
# README
# 
# This is an example script that demonstrated how to implement a Hiero Flipbook
################################################################################

class FlipbookHiero(flipbooking.FlipbookApplication):
  """This is an example implementation of how to deal with implementing a 
     flipbook application other than FrameCycler for Nuke - in this case, Hiero."""
  def __init__(self):
    pass

  def name(self):
    return "Hiero Player"

  def path(self):
    return os.path.join(sys.executable)

  def cacheDir(self):
    return os.environ["NUKE_TEMP_DIR"]

  def run(self, filename, frameRanges, views, options):
    sequence_interval = str(frameRanges.minFrame())+"-"+str(frameRanges.maxFrame())

    #if nuke.env['WIN32']:
    #  filename = filename.replace("/", "\\")
    
    # figure out our options
    burnIn = options.get("burnIn",False)
    
    # The lut string coming from the renderdialog is in the form 'inputcolourspace-outputcolourspace' or just 'colourspace' if they're the same.
    # Check if the string ends with sRGB or rec709 to determine what should be set on the viewer.
    lut = options.get("lut", "")
    selectedLUT = "None"
    if lut.endswith("sRGB"):
      selectedLUT = "sRGB"
    elif lut.endswith("rec709"):
      selectedLUT = "rec709"
    elif lut.endswith("rec1886"):
      selectedLUT = "rec1886"

    args = [self.path()]

    args.append("--player")
    args.append("-q")
    args.append("--script")
    pathToFlipbook = os.path.join(nuke.pluginPath()[0],"flipbook_hiero_launch.py")
    args.append(pathToFlipbook)
    args.append(str(filename))
    args.append(str(frameRanges.minFrame()))
    args.append(str(frameRanges.maxFrame()))
    args.append(selectedLUT)
    args.append(str(burnIn))
    self._hieroProcess = subprocess.Popen(args, shell=False, stdout=subprocess.PIPE, stderr=subprocess.PIPE)

  def capabilities(self):
    return { 
      'roi': False,
      'proxyScale': True,
      'crop': True,
      'canPreLaunch': True,
      'arbitraryChannels': True,
      'maximumViews' : 2, 
      'fileTypes' : ["j2k","jpt","jp2","dpx","cin","cineon","jpeg","jpg","rla",
      "rpf","yuv","exr","openexr","sxr","tif","tiff","sm","tex","tx","tdl",
      "shd","targa","tga","tpic","rgbe","hdr","iff","png","z","zfile","sgi",
      "bw","rgb","rgba","*mraysubfile*","movieproc","stdinfb","aiff","aif",
      "aifc","wav","snd","au","mov","avi","mp4","m4v","dv","arri","ari","r3d",
      "dng"]

    }

flipbooking.register(FlipbookHiero())

from nukescripts.renderdialog import *

# This overrides Nuke's default renderdialog.py, without having to modify the Nuke install.
def _getIntermediatePath(self):
  """Get the path for the temporary files. May be filled in using printf syntax."""
  flipbooktmp=""
  if flipbooktmp == "":
    try:
      flipbooktmp = self._selectedFlipbook().cacheDir()
    except:
      try:
        flipbooktmp = os.environ["NUKE_DISK_CACHE"]
      except:
        flipbooktmp = nuke.value("preferences.DiskCachePath")

  if len(self._selectedViews()) > 1:
    flipbookFileNameTemp = "nuke_tmp_flip.%04d.%V." + self._getIntermediateFileType()
  else:
    # Here, we're replacing the 'nuke_tmp_flip' string which is used by FrameCycler. We want unique render names.
    flipbookFileNameTemp = "%s_flip_%i.%s." % (self._node.name(),int(time.time()),'%04d') + self._getIntermediateFileType()
  flipbooktmpdir = os.path.join(flipbooktmp, "flipbook")
  if not os.path.exists(flipbooktmpdir):
    os.mkdir(flipbooktmpdir)

  if not os.path.isdir(flipbooktmpdir):
    raise RuntimeError("%s already exists and is not a directory, please delete before flipbooking again" % flipbooktmpdir)
  flipbooktmp = os.path.join(flipbooktmpdir, flipbookFileNameTemp)

  if nuke.env['WIN32']:
    flipbooktmp = re.sub(r"\\", "/", str(flipbooktmp))
  return flipbooktmp

def _getOptions(self, nodeToFlipbook):
  options = {
  }

  try:
    options['pixelAspect'] = float(nuke.value(nodeToFlipbook.name()+".pixel_aspect"))
  except:
    pass

  try:
    f = nodeToFlipbook.format()
    options['dimensions'] = { 'width' : f.width(), 'height' : f.height() }
  except:
    pass

  # LUT
  if not self._burnInLUT.value():
    inputColourspace = "linear"
    outputColourspace = "linear"
    # Check if we have a different than linear input
    if self._node.Class() == "Read" or self._node.Class() == "Write":
      lut = self._node.knob("colorspace").value()
      # Might be in the format of "default (foo)", if so, get at "foo".
      if lut[:7] == "default":
        lut = lut[9:-1]
      inputColourspace = lut

    # Check our output
    lut = self._getLUT()
    if lut != "None":
      outputColourspace = lut

    if inputColourspace == outputColourspace:
      options["lut"] = inputColourspace
    else:
      options["lut"] = inputColourspace + "-" + outputColourspace
  # AUDIO
  audioTrack = self._getAudio()
  if audioTrack != "":
    options["audio"] = audioTrack

  # ROI
  if self._useRoi.value():
    roi = self._roi.toDict()
    if (roi["r"] - roi["x"] > 0) and (roi["t"] - roi["y"] > 0):
      options["roi"] = bboxToTopLeft(int(nuke.value(nodeToFlipbook.name()+".actual_format.height")), roi)

  # Burn in the LUT option
  if self._burnInLUT.value():
    options["burnIn"] = self._burnInLUT.value()

  return options

# Insert unique flipbook image path methods for Hiero
FlipbookDialog._getIntermediatePath = _getIntermediatePath
FlipbookDialog._getOptions = _getOptions

# Fix audio read node selection in the flipbook dialog
def fixedAudioFlipbookDialogPostKnobs(self):
  
  # get the old method to do all the heavy lifting
  oldPostKnobs(self)
  
  # make audio reads select themselves in the item list
  if self._node.Class() == "AudioRead":
    index = 0
    for a in self._audioSource.values():
      if a == self._node.name():
        self._audioSource.setValue(index)
        break
      index += 1
        
oldPostKnobs = FlipbookDialog._addPostKnobs
FlipbookDialog._addPostKnobs = fixedAudioFlipbookDialogPostKnobs
